#ifndef BLE_ESS_H__
#define BLE_ESS_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

/**@brief   Macro for defining a ble_hrs instance.
 *
 * @param   _name   Name of the instance.
 * @hideinitializer
 */
#define BLE_ESS_DEF(_name)                                                                          \
static ble_ess_t _name;                                                                             \
NRF_SDH_BLE_OBSERVER(_name ## _obs,                                                                 \
                     BLE_HRS_BLE_OBSERVER_PRIO,                                                     \
                     ble_ess_on_ble_evt, &_name)


// BEHNAM
// BLE_UUID_ENVIRONMENT_SENSING_SERVICE 00000000-0000-1000-8000-00805F9B34FB

#define BLE_UUID_ENVIRONMENT_SENSING_SERVICE			0x181A     /**< Environmental Sensing Service UUID. */
#define BLE_UUID_TEMPERATURE_CHAR									0x2A6E     /**< Temperature characteristic UUID. */
#define BLE_UUID_HUMIDITY_CHAR										0x2A6F     /**< Humidity characteristic UUID. */

#define BLE_UUID_ENVIRONMENT_SENSING_SERVICE_BASE		{0xFB, 0x34, 0x9B, 0x5F, 0x80, 0x00, 0x00, 0x80, \
																										 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
// BEHNAM-End
																					

																					
/**@brief Custom Service event type. */
typedef enum
{
    BLE_ESS_EVT_NOTIFICATION_ENABLED,                             /**< Custom value notification enabled event. */
    BLE_ESS_EVT_NOTIFICATION_DISABLED,                             /**< Custom value notification disabled event. */
    BLE_ESS_EVT_DISCONNECTED,
    BLE_ESS_EVT_CONNECTED
} ble_ess_evt_type_t;

/**@brief Custom Service event. */
typedef struct
{
    ble_ess_evt_type_t evt_type;                                  /**< Type of event. */
} ble_ess_evt_t;

// Forward declaration of the ble_ess_t type.
typedef struct ble_ess_s ble_ess_t;


/**@brief Custom Service event handler type. */
typedef void (*ble_ess_evt_handler_t) (ble_ess_t * p_bas, ble_ess_evt_t * p_evt);

/**@brief ENVIRONMENT SENSING Service init structure. This contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_ess_evt_handler_t         evt_handler;                    /**< Event handler to be called for handling events in the Custom Service. */
    int16_t                       initial_temperature_value;           /**< Initial custom value */
    ble_srv_cccd_security_mode_t  ess_meas_char_attr_md;     /**< Initial security level for Custom characteristics attribute */
} ble_ess_init_t;

/**@brief Custom Service structure. This contains various status information for the service. */
struct ble_ess_s
{
    ble_ess_evt_handler_t         evt_handler;                    /**< Event handler to be called for handling events in the Custom Service. */
    uint16_t                      service_handle;                 /**< Handle of Custom Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t      ess_meas_handles;           		/**< Handles related to the Custom Value characteristic. */
    uint16_t                      conn_handle;                    /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
    uint8_t                       uuid_type; 
};

/**@brief Function for initializing the Custom Service.
 *
 * @param[out]  p_ess       Custom Service structure. This structure will have to be supplied by
 *                          the application. It will be initialized by this function, and will later
 *                          be used to identify this particular service instance.
 * @param[in]   p_ess_init  Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
uint32_t ble_ess_init(ble_ess_t * p_ess, const ble_ess_init_t * p_ess_init);

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @details Handles all events from the BLE stack of interest to the Battery Service.
 *
 * @note 
 *
 * @param[in]   p_ess      Custom Service structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_ess_on_ble_evt( ble_evt_t const * p_ble_evt, void * p_context);

/**@brief Function for updating the custom value.
 *
 * @details The application calls this function when the cutom value should be updated. If
 *          notification has been enabled, the custom value characteristic is sent to the client.
 *
 * @note 
 *       
 * @param[in]   p_bas          Custom Service structure.
 * @param[in]   Custom value 
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */

uint32_t ble_ess_custom_value_update(ble_ess_t * p_ess, uint8_t custom_value);

#endif // BLE_ESS_H__
